/*-
 * Copyright (C)2017 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: pgoblin_dbms.h,v 1.17 2017/07/04 22:36:14 babolo Exp $
 */

#ifndef _PGOBLIN_DBMS_H_
#define _PGOBLIN_DBMS_H_

__BEGIN_DECLS


/*****************************************************************************************************
 *                                                                                                   *
 *                                   pgoblin_conn                                                    *
 *                                                                                                   *
 *****************************************************************************************************/
typedef struct pgoblin_conn {
    BLIN_flag        flags;
#   define           PGOBLIN_DB_TYPE      0x00000F
#   define           PGOBLIN_INT64TIME    0x010000 /* DB known to have int64 timestamp               */
#   define           PGOBLIN_FLOATTIME    0x020000 /* DB known to have float timestamp               */
#   define           PGOBLIN_NEEDINTIME   0x040000 /* Ask DB for timestamp format                    */
#   define           PGOBLIN_CLOSECONN    0x080000 /* Close this connection on exit                  */
#   define           PGOBLIN_PARMIN       0x100000 /* Parms in dbname                                */
#   define           PGOBLIN_PARMDBFREE   0x200000 /* Free DB parms when pgoblin_conn destoyed       */
/*  define           PGOBLIN_ERROR_SENSOR 0x400000    Eror reaction                                  */
/*  define           PGOBLIN_ERROR_SENSIB 0x800000    Immediate error reaction                       */
/*                   BLINflag                         copied from pgoblin_main                       */
    u_char           intran;   /* Transaction level */
    const char      *dbname;   /* For libpq         */
    const char      *host;     /* For libpq         */
    const char      *port;     /* For libpq         */
    const char      *username; /* For libpq         */
    void            *conn;
} pgoblin_conn;

typedef struct pgoblin_dbases {
    BLIN_flag   flags;
#   define PGOBLIN_DB_PREPARE  0x800000
#   define PGOBLIN_DB_COPYIN   0x400000
#   define PGOBLIN_DB_COPYOUT  0x200000
    char           id[PGOBLIN_STRING_ID_LEN];
    char         name[PGOBLIN_STRING_ID_LEN];
    int        (*init)(BLIN_flag flags);
    int        (*fini)();
    int    (*shurecon)(pgoblin_conn *db);
    int       (*query)(pgoblin_conn *db, pgoblin_mode **res, char *qry, u_char cond);
    int     (*prepare)(pgoblin_conn *db, pgoblin_mode **res, char *qry);
    int     (*execute)( pgoblin_conn *db, pgoblin_mode **res, const void *qp
                      , int cnt, const char * const *val, const int *len, u_char cond
                      );
    int     (*resinfo)(int kind, pgoblin_mode *res);
    char   *(*subinfo)(int kind, pgoblin_mode *res, int sub);
    int     (*valinfo)(int kind, pgoblin_mode *res, int row, int col);
    char  *(*getvalue)(pgoblin_mode *res, int row, int col);
    int   (*getstream)(pgoblin_conn *db, pgoblin_mode **res, int quant, u_char cond);
    int     (*getcopy)(pgoblin_conn *db, void **buf);
    int     (*putcopy)(pgoblin_conn *db, const void *buf, int len);
    int   (*endstream)(pgoblin_conn *db, u_char cond);
    int       (*tarry)(pgoblin_conn *db, int timeout);
    char  *(*notifies)(pgoblin_conn *db);
    int      (*finish)(u_int32_t flags, int kind, void **entity);
    int       (*clear)(pgoblin_mode **res);
} pgoblin_dbases;

int   pgoblin_db_shurecon  __P((pgoblin_conn *db));
int   pgoblin_db_query     __P((pgoblin_conn *db, pgoblin_mode **res, char *qry, u_char cond));
int   pgoblin_db_prepare   __P((pgoblin_conn *db, pgoblin_mode **res, char *qry));
int   pgoblin_db_execute   __P(( pgoblin_conn *db
                               , pgoblin_mode **rsv
                               , const void *qp
                               , int cnt
                               , const char * const *val
                               , const int *len
                               , u_char cond
                              ));
int   pgoblin_db_resinfo   __P((int kind, pgoblin_mode *rsv));
char *pgoblin_db_subinfo   __P((int kind, pgoblin_mode *rsv, int sub));
int   pgoblin_db_valinfo   __P((int kind, pgoblin_mode *rsv, int row, int col));
char* pgoblin_db_getvalue  __P((pgoblin_mode *rsv, int row, int col));
int   pgoblin_db_getstream __P((pgoblin_conn *db, pgoblin_mode **rsv, int quant, u_char cond));
int   pgoblin_db_getcopy   __P((pgoblin_conn *db, void **buf));
int   pgoblin_db_putcopy   __P((pgoblin_conn *db, const void *buf, int len));
int   pgoblin_db_endstream __P((pgoblin_conn *db, u_char cond));
int   pgoblin_db_tarry     __P((pgoblin_conn *db, int timeout));
char* pgoblin_db_notifies  __P((pgoblin_conn *db));
int   pgoblin_db_finish    __P((u_int32_t flags, int kind, void **entity));
int   pgoblin_db_clear     __P((pgoblin_mode **rsv));

__END_DECLS

#endif /* !_PGOBLIN_DBMS_H_ */
